package org.apache.xindice.core;

import org.apache.xindice.core.data.Key;
import org.apache.xindice.core.data.Record;
import org.apache.xindice.util.Configuration;

import org.w3c.dom.*;

import java.util.Map;

/**
 * Observer for Xindice DB activities
 */
public abstract class DBObserver
{
    private static final DBObserver NOOP = new DBObserver()
    {
        public void setDatabaseConfig( 
        Database db, Map collections, Configuration cfg ) {}

        public void setCollectionConfig( 
        Collection col, Configuration cfg ) {}

        public void flushDatabaseConfig( 
        Database db, Configuration cfg ) {}
    
        public void dropCollection( Collection col ) 
        throws DBException {}

        public void putDocument( 
        Collection col, Key key, Document document, boolean create ) 
        throws DBException {}

        public void loadDocument( 
            Collection col, Record record, Document document ) 
        throws DBException {}
        
        public void dropDocument( Collection col, Key key ) 
        throws DBException {}
    };

    private static DBObserver instance = NOOP;

    /**
     * Sets the default observer instance
     */ 
    public static void setInstance( DBObserver obs )
    { 
        instance = (null == obs) ? NOOP : obs; 
    }

    /**
     * Returns the observer instance, must be non-null
     */    
    public static DBObserver getInstance()
    { 
        return instance; 
    }
    
    /**
     * Called after Database.setConfig()
     */    
    public abstract void setDatabaseConfig( 
        Database db, Map collections, Configuration cfg );

    /**
     * Called after Collection.setConfig()
     */    
    public abstract void setCollectionConfig( 
        Collection col, Configuration cfg );

    /**
     * Called after Database.flushConfig()
     */
    public abstract void flushDatabaseConfig( 
        Database db, Configuration cfg );
    
    /**
     * Called before Collection.drop()
     */
    public abstract void dropCollection( Collection col ) 
    throws DBException;

    /**
     * Called after Collection.putDocument()
     */    
    public abstract void putDocument( 
        Collection col, Key key, Document document, boolean create ) 
    throws DBException;

    /**
     * Called after Collection.getDocument()
     */    
    public abstract void loadDocument( 
        Collection col, Record record, Document document ) 
    throws DBException;

    /**
     * Called before Collection.remove(key)
     */
    public abstract void dropDocument( Collection col, Key key ) 
    throws DBException;
    
}

