/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: UnitTests.java,v 1.10 2003/02/06 04:56:39 kstaken Exp $
 */

package org.apache.xindice;

import org.apache.xindice.client.xmldb.DatabaseImplTest;
import org.apache.xindice.client.xmldb.ResourceIteratorImplTest;
import org.apache.xindice.core.data.KeyTest;
import org.apache.xindice.core.data.ValueTest;
import org.apache.xindice.core.filer.*;
import org.apache.xindice.tools.XMLToolsTest;
import org.apache.xindice.util.ConfigurationTest;
import org.apache.xindice.core.meta.MetaTest;

import junit.framework.Test;
import junit.framework.TestSuite;
import junitx.extensions.TestSetup;

import java.io.File;

/**
 * @version $Revision: 1.10 $, $Date: 2003/02/06 04:56:39 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 * @author Kimbro Staken <kstaken@xmldatabases.org>
 */
public class UnitTests {

   public static Test suite() {
      TestSuite suite = new TestSuite("Xindice unit tests");

      suite.addTest(new TestSuite(KeyTest.class));
      suite.addTest(new TestSuite(ValueTest.class));

      suite.addTest(new FilerTestSetup(new TestSuite(FilerTestCase.class), new BTreeFiler()));
      suite.addTest(new FilerTestSetup(new TestSuite(FilerTestCase.class), new FSFiler()));
      // todo see what to do with the HashFiler
      suite.addTest(new FilerTestSetup(new TestSuite(FilerTestCase.class), new HashFiler()));
      suite.addTest(new FilerTestSetup(new TestSuite(FilerTestCase.class), new MemFiler()));

      // If the Berkeley filer was built in then test it too.
      try {
         Class filerClass = Class.forName("org.apache.xindice.core.filer.BerkeleyBTreeFiler");
         Filer filer = (Filer) filerClass.newInstance();
         suite.addTest(new FilerTestSetup(new TestSuite(FilerTestCase.class), filer));
      }
      catch (Exception e) {
         // If the class wasn't built just ignore it.
      }


      suite.addTest(new TestSuite(DatabaseImplTest.class));
      suite.addTest(new TestSuite(ResourceIteratorImplTest.class));

      suite.addTest(new TestSuite(XMLToolsTest.class));

      suite.addTest(new TestSuite(ConfigurationTest.class));
      
      suite.addTest(new TestSuite(MetaTest.class));

      return new TestSetup(suite) {

         private File root = new File(FilerTestCase.TEST_COLLECTION_NAME);

         public void setUp() {
            // Create the directory to hold the tests.
            root.mkdir();
         }

         public void tearDown() {
            String[] files = root.list();
            for (int i = 0; i < files.length; i++) {
               new File(root, files[i]).delete();
            }
            root.delete();
         }
      };
   }

}
